/*
 *  Copyright (C) Chaperon. All rights reserved.                               
 *  ------------------------------------------------------------------------- 
 *  This software is published under the terms of the Apache Software License 
 *  version 1.1, a copy of which has been included  with this distribution in 
 *  the LICENSE file.                                                         
 */

package net.sourceforge.chaperon.parser;

import java.util.Stack;
import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import org.xml.sax.InputSource;
import net.sourceforge.chaperon.helpers.Decoder;
/*import net.sourceforge.chaperon.parser.output.ASTFactory;
import net.sourceforge.chaperon.parser.output.ASTLeaf;*/

/**
 * The scanner tokenize a text
 *
 * @author Stephan Michels
 * @version CVS $Id: Scanner.java,v 1.9 2002/05/06 16:22:56 benedikta Exp $
 */
public class Scanner
{

  private RegexParser _regexparser = new RegexParser();

  // The text
  private String      _text;
  private char[]      _textAsChars;

  // Parser table of the grammar
  private ParserTable _table;

  // Current position
  private int         _position = 0;

  private int         _token;
  private int         _textstart;
  private int         _textlength;
  private boolean     _ignorable;

  // If the scaner produce debug messages
  //private boolean     _debugmessages = false;

  /**
   * Constructs a scanner with the token from a parser table
   *
   * @param table Parser table
   */
  public Scanner(ParserTable table)
  {
    _table = table;
  }

  /**
   * Parse the text from an inputsource
   *
   * @param input Inputsource
   */
  public void parse(InputSource input)
  {
    InputStreamReader in = new InputStreamReader(input.getByteStream());
    readText(in);
    reset();
  }

  /*
   * Parse the text from an inputsource
   *
   * @param input Inputsource
   */
  public void parse(InputSource input, String encoding) throws UnsupportedEncodingException
  { 
    InputStreamReader in = new InputStreamReader(input.getByteStream(), encoding);
    readText(in);
    reset();
  }

  /**
   * Parse the text from an inputstream
   *
   * @param input Inputstream
   */
  public void parse(InputStream input)
  { 
    InputStreamReader in = new InputStreamReader(input);
    readText(in);
    reset();
  }


  /**
   * Parse the text from an inputstream
   *
   * @param input Inputstream
   */
  public void parse(InputStream input, String encoding) throws UnsupportedEncodingException
  { 
    InputStreamReader in = new InputStreamReader(input, encoding);
    readText(in);
    reset();
  }

  /**
   * Returns the text to parse
   *
   * @return Text
   */
  public String getText()
  {
    return _text;
  }

  /**
   * Returns the text as sequence of characters
   *
   * @return Sequence of characters
   */
  public char[] getTextAsChars()
  {
    return _textAsChars;
  }

  /**
   * The scanner should go backward to the first position
   * in the text
   */
  private void reset()
  {
    _position = 0;
  }

  /**
   * Read the text from an inputsource
   *
   * @param input Inputsource
   */
  private void readText(InputStreamReader in)
  {
    try
    {
      if (in == null)
      {
        _text = null;
        _textAsChars = null;
        return;
      }
      StringBuffer buffer = new StringBuffer();
      int c;

      while (true)
      {
        c = in.read();
        if (c == -1)
          break;
        buffer.append((char)c);
      }
      _text = buffer.toString();

      if ((_text != null) && (_text.length() > 0))
      {
        _textAsChars = _text.toCharArray();
      }
      else
      {
        _text = null;
        _textAsChars = null;
      }

    }
    catch (IOException e)
    {
      System.err.println("RegexScanner:Error at the reading from the InputStream");
      _text = null;
      _textAsChars = null;
    }
  }

  /*
   * Parse next token if it compares with the automate at index
   *
   * @param index Index of the token
   * @param document Document, in which the element should created
   *
   * @return Index of the element in the document
   */
  public void readNextToken(int state)
  {
    if (_textAsChars == null)
      return ;

    int tokenindex, nextposition;
    
    // try every token, which does't process an error
    for(tokenindex = 0; tokenindex < _table.getTerminalSymbolCount(); tokenindex++)
      if (!_table.isErrorAction(state, tokenindex))
      {
        //System.out.println("Trying "+_table.getTerminalSymbol(tokenindex));

        nextposition = _regexparser.parse(_textAsChars,
                                          _table.getTokenDefinition(tokenindex), _position);
        if (nextposition > 0) // if the token was found
        {
          //System.out.println("Found "+_table.getTerminalSymbol(tokenindex));
          _token = tokenindex;
          _textstart = _position;
          _textlength = nextposition - _position;
          _ignorable = false;

          _position = nextposition;

          return;
        }
      }

    // try the ignorable tokens
    for(tokenindex = 0; tokenindex < _table.getIgnorableTokenCount(); tokenindex++)
    {
      //System.out.println("Trying "+_table.getIgnorableTokenSymbol(tokenindex));
      nextposition = _regexparser.parse(_textAsChars, _table.getIgnorableTokenDefinition(tokenindex), _position);
      if (nextposition > 0)
      {
        //System.out.println("Found "+_table.getIgnorableTokenSymbol(tokenindex));
        _token = tokenindex;
        _textstart = _position;
        _textlength = nextposition - _position;
        _ignorable = true;

        _position = nextposition;

        return;
      }
    }

    // try every token, which does't process an error
    for(tokenindex = 0; tokenindex < _table.getTerminalSymbolCount(); tokenindex++)
      if (!_table.isErrorAction(state, tokenindex))
      {
        //System.out.println("Trying "+_table.getTerminalSymbol(tokenindex));
        nextposition = _regexparser.parse(_textAsChars,
                                          _table.getTokenDefinition(tokenindex), _position);
        if (nextposition > 0) // if the token was found
        {
          //System.out.println("Found "+_table.getTerminalSymbol(tokenindex));
          _token = tokenindex;
          _textstart = _position;
          _textlength = nextposition - _position;
          _ignorable = false;

          _position = nextposition;

          return;
        }
      }

    // try every other token
    for(tokenindex = 0; tokenindex < _table.getTerminalSymbolCount(); tokenindex++)
      if (_table.isErrorAction(state, tokenindex))
      {
        //System.out.println("Trying "+_table.getTerminalSymbol(tokenindex));
        nextposition = _regexparser.parse(_textAsChars,
                                          _table.getTokenDefinition(tokenindex), _position);
        if (nextposition > 0) // if the token was found
        {
          //System.out.println("Found "+_table.getTerminalSymbol(tokenindex));
          _token = tokenindex;
          _textstart = _position;
          _textlength = nextposition - _position;
          _ignorable = false;

          _position = nextposition;

          return;
        }
      }


    _token = -1;
    _textstart = -1;
    _textlength = -1;
    _ignorable = false;
    return;
  }

  public int getTokenIndex()
  {
    return _token;
  }

  public int getTextStart()
  {
    return _textstart;
  }

  public int getTextLength()
  {
    return _textlength;
  }

  public boolean isIgnorable()
  {
    return _ignorable;
  }

  /**
   * Return true if the scanner has reached the end of file
   *
   * @return True if the scanner has reached the end of file
   */
  public boolean isEOF()
  {
    return ((_textAsChars == null) || (_position >= _textAsChars.length));
  }

  /**
   * Returns the current position
   *
   * @return Current position
   */
  public int getPosition()
  {
    return _position;
  }

  /**
   * Returns the last position in the text
   *
   * @return Last position in the text
   */
  public int getLastPosition()
  {
    return _textAsChars.length;
  }

  /**
   * Returns the current column number
   *
   * @return Current column number
   */
  public int getColumnNumber()
  {
    if (_text == null)
      return 0;
    return _position
           - Math.max(_text.lastIndexOf("\n", _position - 1),
                      _text.lastIndexOf("\r", _position - 1));
  }

  /**
   * Return the current line number
   *
   * @return Current line number
   */
  public int getLineNumber()
  {
    if (_text == null)
      return 0;
    int linenumber = 1;

    for (int i = 0; i < _position; i++)
      if (_text.charAt(i) == '\n')
        linenumber++;
    return linenumber;
  }

  /**
   * Sets if the scanner should output debug informations
   *
   * @param debug true, if the scanner should output debug informations
   */
  /*public void setDebug(boolean debug)
  {
    _debugmessages = debug;
    _regexparser.setDebug(debug);
  }*/

  /**
   * Print the current row, and a mark to the current position
   */
  public void printRow()
  {
    int from = Math.max(_text.lastIndexOf("\n", _position - 1),
                        _text.lastIndexOf("\r", _position - 1));
    int to = _text.indexOf("\n", _position);

    if (to == -1)
      to = _text.indexOf("\r", _position);
    else if (_text.indexOf("\r", _position) != -1)
      to = Math.min(to, _text.indexOf("\r", _position));
    if (to == -1)
      to = _textAsChars.length;

    System.out.println(_text.substring(from + 1, to));

    for (int i = from + 1; i < _position; i++)
      System.out.print(" ");
    System.out.println("^");

    int column = _position - from;
    int row = 1;

    for (int i = 0; i < _position; i++)
      if (_text.charAt(i) == '\n')
        row++;
    System.out.println("(Row " + row + " / Column " + column + ")");
  }

  /**
   * Prints the current position
   */
  public void printPosition()
  {
    int from = Math.max(_text.lastIndexOf("\n", _position - 1),
                        _text.lastIndexOf("\r", _position - 1));

    int column = _position - from;
    int row = 1;

    for (int i = 0; i < _position; i++)
      if (_text.charAt(i) == '\n')
        row++;
    System.out.print("(Row " + row + " / column " + column + ")");
  }

  /**
   * Print the rest of the input from the current position
   */
  public void printRest()
  {
    System.out.println(Decoder.decode(_text.substring(_position,
            Math.min(_position + 20, _text.length() - 1))));
  }
}
